import mime from "mime";
import { Readable } from "stream";
import { call as callEndpoint } from "../core/endpoint/dev/index.js";
import {
  collectErrorMetadata,
  fixViteErrorMessage,
  getViteErrorPayload
} from "../core/errors.js";
import { error, info, warn } from "../core/logger/core.js";
import * as msg from "../core/messages.js";
import { appendForwardSlash } from "../core/path.js";
import { getParamsAndProps, GetParamsAndPropsError } from "../core/render/core.js";
import { preload, ssr } from "../core/render/dev/index.js";
import { RouteCache } from "../core/render/route-cache.js";
import { createRequest } from "../core/request.js";
import { createRouteManifest, matchAllRoutes } from "../core/routing/index.js";
import { createSafeError, resolvePages } from "../core/util.js";
import notFoundTemplate, { subpathNotUsedTemplate } from "../template/4xx.js";
const BAD_VITE_MIDDLEWARE = [
  "viteIndexHtmlMiddleware",
  "vite404Middleware",
  "viteSpaFallbackMiddleware"
];
function removeViteHttpMiddleware(server) {
  for (let i = server.stack.length - 1; i > 0; i--) {
    if (BAD_VITE_MIDDLEWARE.includes(server.stack[i].handle.name)) {
      server.stack.splice(i, 1);
    }
  }
}
function truncateString(str, n) {
  if (str.length > n) {
    return str.substring(0, n) + "&#8230;";
  } else {
    return str;
  }
}
function writeHtmlResponse(res, statusCode, html) {
  res.writeHead(statusCode, {
    "Content-Type": "text/html; charset=utf-8",
    "Content-Length": Buffer.byteLength(html, "utf-8")
  });
  res.write(html);
  res.end();
}
async function writeWebResponse(res, webResponse) {
  const { status, headers, body } = webResponse;
  let _headers = {};
  if ("raw" in headers) {
    for (const [key, value] of Object.entries(headers.raw())) {
      res.setHeader(key, value);
    }
  } else {
    _headers = Object.fromEntries(headers.entries());
  }
  res.writeHead(status, _headers);
  if (body) {
    if (Symbol.for("astro.responseBody") in webResponse) {
      let stream = webResponse[Symbol.for("astro.responseBody")];
      for await (const chunk of stream) {
        res.write(chunk.toString());
      }
    } else if (body instanceof Readable) {
      body.pipe(res);
      return;
    } else if (typeof body === "string") {
      res.write(body);
    } else {
      const reader = body.getReader();
      while (true) {
        const { done, value } = await reader.read();
        if (done)
          break;
        if (value) {
          res.write(value);
        }
      }
    }
  }
  res.end();
}
async function writeSSRResult(webResponse, res) {
  return writeWebResponse(res, webResponse);
}
async function handle404Response(origin, config, req, res) {
  const pathname = decodeURI(new URL(origin + req.url).pathname);
  const html = notFoundTemplate({
    statusCode: 404,
    title: "Not found",
    tabTitle: "404: Not Found",
    pathname
  });
  writeHtmlResponse(res, 404, html);
}
async function handle500Response(viteServer, origin, req, res, err) {
  res.on("close", () => setTimeout(() => viteServer.ws.send(getViteErrorPayload(err)), 200));
  if (res.headersSent) {
    res.end();
  } else {
    writeHtmlResponse(
      res,
      500,
      `<title>${err.name}</title><script type="module" src="/@vite/client"><\/script>`
    );
  }
}
function getCustom404Route(config, manifest) {
  const relPages = resolvePages(config).href.replace(config.root.href, "");
  const pattern = new RegExp(`${appendForwardSlash(relPages)}404.(astro|md)`);
  return manifest.routes.find((r) => r.component.match(pattern));
}
function log404(logging, pathname) {
  info(logging, "serve", msg.req({ url: pathname, statusCode: 404 }));
}
function baseMiddleware(config, logging) {
  const site = config.site ? new URL(config.base, config.site) : void 0;
  const devRoot = site ? site.pathname : "/";
  return function devBaseMiddleware(req, res, next) {
    var _a;
    const url = req.url;
    const pathname = decodeURI(new URL(url, "http://vitejs.dev").pathname);
    if (pathname.startsWith(devRoot)) {
      req.url = url.replace(devRoot, "/");
      return next();
    }
    if (pathname === "/" || pathname === "/index.html") {
      log404(logging, pathname);
      const html = subpathNotUsedTemplate(devRoot, pathname);
      return writeHtmlResponse(res, 404, html);
    }
    if ((_a = req.headers.accept) == null ? void 0 : _a.includes("text/html")) {
      log404(logging, pathname);
      const html = notFoundTemplate({
        statusCode: 404,
        title: "Not found",
        tabTitle: "404: Not Found",
        pathname
      });
      return writeHtmlResponse(res, 404, html);
    }
    next();
  };
}
async function handleRequest(routeCache, viteServer, logging, manifest, config, req, res) {
  var _a;
  const reqStart = performance.now();
  const origin = `${viteServer.config.server.https ? "https" : "http"}://${req.headers.host}`;
  const buildingToSSR = config.output === "server";
  const url = new URL(origin + ((_a = req.url) == null ? void 0 : _a.replace(/(index)?\.html$/, "")));
  const pathname = decodeURI(url.pathname);
  url.pathname = config.base.substring(0, config.base.length - 1) + url.pathname;
  if (!buildingToSSR && pathname !== "/_image") {
    const allSearchParams = Array.from(url.searchParams);
    for (const [key] of allSearchParams) {
      url.searchParams.delete(key);
    }
  }
  let body = void 0;
  if (!(req.method === "GET" || req.method === "HEAD")) {
    let bytes = [];
    await new Promise((resolve) => {
      req.setEncoding("utf-8");
      req.on("data", (bts) => bytes.push(bts));
      req.on("end", resolve);
    });
    body = new TextEncoder().encode(bytes.join("")).buffer;
  }
  const request = createRequest({
    url,
    headers: buildingToSSR ? req.headers : new Headers(),
    method: req.method,
    body,
    logging,
    ssr: buildingToSSR,
    clientAddress: buildingToSSR ? req.socket.remoteAddress : void 0
  });
  async function matchRoute() {
    const matches = matchAllRoutes(pathname, manifest);
    for await (const maybeRoute of matches) {
      const filePath2 = new URL(`./${maybeRoute.component}`, config.root);
      const preloadedComponent = await preload({ astroConfig: config, filePath: filePath2, viteServer });
      const [, mod] = preloadedComponent;
      const paramsAndPropsRes = await getParamsAndProps({
        mod,
        route: maybeRoute,
        routeCache,
        pathname,
        logging,
        ssr: config.output === "server"
      });
      if (paramsAndPropsRes !== GetParamsAndPropsError.NoMatchingStaticPath) {
        return {
          route: maybeRoute,
          filePath: filePath2,
          preloadedComponent,
          mod
        };
      }
    }
    if (matches.length) {
      warn(
        logging,
        "getStaticPaths",
        `Route pattern matched, but no matching static path found. (${pathname})`
      );
    }
    log404(logging, pathname);
    const custom404 = getCustom404Route(config, manifest);
    if (custom404) {
      const filePath2 = new URL(`./${custom404.component}`, config.root);
      const preloadedComponent = await preload({ astroConfig: config, filePath: filePath2, viteServer });
      const [, mod] = preloadedComponent;
      return {
        route: custom404,
        filePath: filePath2,
        preloadedComponent,
        mod
      };
    }
    return void 0;
  }
  let filePath;
  try {
    const matchedRoute = await matchRoute();
    if (!matchedRoute) {
      return handle404Response(origin, config, req, res);
    }
    const { route, preloadedComponent, mod } = matchedRoute;
    filePath = matchedRoute.filePath;
    const paramsAndPropsRes = await getParamsAndProps({
      mod,
      route,
      routeCache,
      pathname,
      logging,
      ssr: config.output === "server"
    });
    const options = {
      astroConfig: config,
      filePath,
      logging,
      mode: "development",
      origin,
      pathname,
      route,
      routeCache,
      viteServer,
      request
    };
    if (route.type === "endpoint") {
      const result = await callEndpoint(options);
      if (result.type === "response") {
        await writeWebResponse(res, result.response);
      } else {
        let contentType = "text/plain";
        const filepath = route.pathname || route.segments.map((segment) => segment.map((p) => p.content).join("")).join("/");
        const computedMimeType = mime.getType(filepath);
        if (computedMimeType) {
          contentType = computedMimeType;
        }
        res.writeHead(200, { "Content-Type": `${contentType};charset=utf-8` });
        res.end(result.body);
      }
    } else {
      const result = await ssr(preloadedComponent, options);
      return await writeSSRResult(result, res);
    }
  } catch (_err) {
    const err = fixViteErrorMessage(createSafeError(_err), viteServer, filePath);
    const errorWithMetadata = collectErrorMetadata(err);
    error(logging, null, msg.formatErrorMessage(errorWithMetadata));
    handle500Response(viteServer, origin, req, res, errorWithMetadata);
  }
}
function createPlugin({ config, logging }) {
  return {
    name: "astro:server",
    configureServer(viteServer) {
      let routeCache = new RouteCache(logging);
      let manifest = createRouteManifest({ config }, logging);
      function rebuildManifest(needsManifestRebuild, file) {
        routeCache.clearAll();
        if (needsManifestRebuild) {
          manifest = createRouteManifest({ config }, logging);
        }
      }
      viteServer.watcher.on("add", rebuildManifest.bind(null, true));
      viteServer.watcher.on("unlink", rebuildManifest.bind(null, true));
      viteServer.watcher.on("change", rebuildManifest.bind(null, false));
      return () => {
        removeViteHttpMiddleware(viteServer.middlewares);
        if (config.base !== "/") {
          viteServer.middlewares.stack.unshift({
            route: "",
            handle: baseMiddleware(config, logging)
          });
        }
        viteServer.middlewares.use(async (req, res) => {
          if (!req.url || !req.method) {
            throw new Error("Incomplete request");
          }
          handleRequest(routeCache, viteServer, logging, manifest, config, req, res);
        });
      };
    },
    transform(code, id, opts = {}) {
      if (opts.ssr)
        return;
      if (!id.includes("vite/dist/client/client.mjs"))
        return;
      return code.replace(/\.tip \{[^}]*\}/gm, ".tip {\n  display: none;\n}").replace(/\[vite\]/g, "[astro]");
    }
  };
}
export {
  baseMiddleware,
  createPlugin as default
};
